# $Id: ModFileTron.pm,v 1.14 2004/05/16 23:27:26 uhtu Exp $
# Copyright (c) 2002-2004 Robert Hagemann <bobzilla@gothictech.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModFileTron;

@ModFileTron::ISA = qw(OmniMod);

use strict;

use POSIX;
use URI::Escape;

# XXX: dirty!
use ModChatLog;

my $on = 1;

my $fileposturl = 'http://mimezine.com/cgi-bin/post/';
my $filegeturl = 'http://mimezine.com/dump/';
my $filedumppath = '/usr/local/dump/';
#my $filedumppath = '/usr/local/dumptest/';
my $timefile = "timefile.txt";
my $lasttime = 0;
my $waittime = 5;
my %donefiles = ();
# boolean to tell whether we were scheduled yet
my $scheduled = 0;

# get the last time we checked the dir
open TIMEFILE, $timefile;
while(<TIMEFILE>) {
  my $line = $_;
  if ( defined $line and $line !~ /^\s*$/ ) {
    $lasttime = $line;
    $lasttime =~ s/\s+//sgi;
    last;
  }
}
close TIMEFILE;

sub help {
  return if ! $on;

  if ( defined wantarray ) { # do we have help?
    return 1;
  }

  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  
  $irc->notice( $nick, "Post files at: $fileposturl" );
  $irc->notice( $nick, "[filedump [<maxhits>]]  ->  Dump of files in file store, optionally only get <maxhits> number of results" );
  $irc->notice( $nick, "[filesearch [<maxhits>] <regex>]  ->  Search file store, optionally only get <maxhits> number of results" );
}

sub name() {
  my $this = shift;
  return "filetron";
}

sub handleConnect {
  my $this = shift;
  my $irc = shift;

  # schedule the file checking for filetron functionality
  $irc->schedule($waittime, \&checkFiles, $irc);
  $scheduled = 1;
}

sub handleMessage {
  my $this = shift;
  my $irc = shift;

  return if ! $on;

  if ( ! $scheduled ) {
    $irc->schedule($waittime, \&checkFiles, $irc);
    $scheduled = 1;
  }
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;

  if ( ! $scheduled ) {
    $irc->schedule($waittime, \&checkFiles, $irc);
    $scheduled = 1;
  }

  return if ! &applicable( $message );

  &do_file( $irc, $nick, $message );
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^file/ ) {
    return 1;
  }
  return 0;
}

sub do_file {
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  if ( $message =~ /^\s*filedump\s+(\d+)\s*$/si ) {
    my $maxhits = $1;
    &fileSearch( $irc, $nick, ".", $maxhits );
  }
  elsif ( $message =~ /^\s*filedump\s*$/si ) {
    &fileSearch( $irc, $nick, "." );
  }
  elsif ( $message =~ /^\s*filesearch\s+(\d+)\s+(.+?)\s*$/si ) {
    my $maxhits = $1;
    my $regex = $2;
    &fileSearch( $irc, $nick, $regex, $maxhits );
  }
  elsif ( $message =~ /^\s*filesearch\s+(.+?)\s*$/si ) {
    my $regex = $1;
    &fileSearch( $irc, $nick, $regex );
  }
}

# filetron's file searching
sub fileSearch {
	my ( $irc, $nick, $string, $maxhits) = @_;
	$maxhits = 5 if ( ! defined $maxhits );
		
	# cover bad regexen
  eval { qr/$string/ };
  if ($@) {
    $irc->notice($nick, "pattern '$string' is not a valid regex: $!"); 
    return;
  }

	$irc->notice( $nick, "[Beginning search '$string' Max Hits: $maxhits]" );

	my %filehash = ();

	while( glob $filedumppath . "*" ) {
		my $file = $_;
		my ($filesize,$filetime) = (stat($file))[7,9];
		
		# cover our butts if two files are posted at the same time...
		if ( defined $filehash{ $filetime } ) {
			$filetime .= "\x00$file"; # \x00 is an illegal filename character
		}

		$filehash{ $filetime } = [$file, $filesize];
	}
	
	my $count = 1;
	foreach my $filekey ( reverse sort keys %filehash ) {
		my ($file,$filesize) = @{$filehash{ $filekey }};

		($filekey) = split/\x00/,$filekey; # strip off the filename, if present

		if ( $count > $maxhits ) {
			last;
		}

		# $irc->notice( "file:$_ time:$filetime now:$now" );
		$file =~ s/^.*\/(.*?)$/$1/si;

		if ( $file =~ /$string/si ) {
			# urlencode
			$file = URI::URL->new($file, $filegeturl)->abs();

			# get date
			my $time = strftime( "%D", localtime( $filekey ) );

			# send to channel
			$irc->notice( $nick, $time . " " . $file . " (" . int ($filesize/1024)." kB)");
			$count++;
		}

	}
	$irc->notice( $nick, "[Finished search '$string']" );
}

# filetron's filechecker
sub checkFiles {
	my ($irc) = @_;

	my $now = time;
	# $irc->notice( "checking files, last:$lasttime now:$now" );

        # if the dir hasn't been touched since lasttime, don't bother scanning;
        my $dirtime = (stat $filedumppath)[9];

        # be sure to reschedule ourselves
        if ( $dirtime < $lasttime ) {
          $irc->schedule($waittime, \&checkFiles);
          return;
        }

	while( glob $filedumppath . "*" ) {
		my $file = $_;
		my ($filesize, $filetime) = (stat($file))[7,9];
		# $irc->notice( "file:$_ time:$filetime now:$now" );
		if ( $filetime >= $lasttime ) {
			$file =~ s/^.*\/(.*?)$/$1/si;
			# urlencode
			my $fileuri = URI::URL->new($file, $filegeturl);
			my $fileurl = $fileuri->abs();
      $fileurl =~ s/\(/%28/sgi;
      $fileurl =~ s/\)/%29/sgi;
			my $displaysize = int ($filesize/1024);
			if ( ! defined $donefiles{$file} ) {
			  my $message = '';
			  my $channel = undef;
				my $mfile = $filedumppath . "." . $file;
				message: if ( -e $mfile ) {
				  local $/;
				  open M, "< $mfile" or last message;
				  undef $/;
				  $message = <M>;
				  close M;
          ($message, $channel) = split /\n/, $message;
				  $message = "'$message'";
				}
        if ( defined $channel ) {
					$irc->privmsg( $channel, "$fileurl ($displaysize kB) $message");
					&ModChatLog::saveMessage("omnicron", "$fileurl ($displaysize kB) $message", $channel );

        }
        else {
				  my $chans = OmniHash::getHash( "joinchans" );
				  foreach my $joinchan ( keys %{$chans} ) {
					  $irc->privmsg( $joinchan, "$fileurl ($displaysize kB) $message");
				  }
				  OmniHash::closeAllHashes();
        }
			}
			$donefiles{$file} = 1;
		}
	}

	open TIMEFILE, ">" . $timefile;
	print TIMEFILE $now . "\n";
	close TIMEFILE;
	$lasttime = $now;

	$irc->schedule($waittime, \&checkFiles);
}

1;
