# $Id: ModIs.pm,v 1.11 2007/02/18 14:22:16 bobzilla Exp $
# Copyright (c) 2003-2004 Robert Hagemann <bobzilla@gothictech.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModIs;

@ModIs::ISA = qw(OmniMod);

use strict;

my $on = 1;

my $table = "ISTABLE";
my $SILENCE= ":silence:";

sub help {
  return if ! $on;

  if ( defined wantarray ) { # do we have help?
    return 1;
  }

  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

	$irc->notice( $nick, "[<thing> is <blah>] define thing" );
	$irc->notice( $nick, "[<thing> is also <blah>] another definition" );
	$irc->notice( $nick, "[<thing> is exact <blah>] respond with only the definition" );
	$irc->notice( $nick, "[forget <thing>] forget about thing" );
	$irc->notice( $nick, "[<thing>?] get thing's definition" );
	$irc->notice( $nick, "[<thing>!] get thing's definition" );
}

sub name() {
  my $this = shift;
  return "is";
}

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;
  my $event = shift;

  return if ! $on;

  my $output = &checkAnyMessage( $irc, $nick, $channel, $message, $channel, 0 );
	&ModChatLog::saveMessage('omnicron', $output, $channel )
    if defined $output and $output !~ /^\s*$/si;
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;
  my $event = shift;

  return if ! $on;

  &checkAnyMessage( $irc, $nick, undef, $message, $nick, 1 );
}

sub checkAnyMessage {
  my $irc = shift;
  my $nick = shift;
  my $channel = shift;
  my $message = shift;
  my $sendTo = shift;
  my $isPrivate = shift;

  my $output;

  if ( $message =~ /^(.*?\S+)\s+is\s+(\S+.*?)$/si ) {
    my $thing = $1;
    my $def = $2;

		$thing =~ s/^\s+//sgi;
		$thing =~ s/\s+$//sgi;
		$thing = lc $thing;
		$def =~ s/^\s+//sgi;
		$def =~ s/\s+$//sgi;

    $def .= "::::$nick,";
    $def .= $channel if defined $channel;

		my @words = split /\s+/, $thing;
		if ( scalar @words <= 3 ) {
    	my $old = &getData( $thing );

      if ( defined $old and $old eq $SILENCE and $isPrivate ) {
        # pretend there isn't an old silence, this is a private
        $old = undef;
      }
      
			if ( defined $old ) {
				if ( $def =~ /^also\s+(\S+.*?)$/si ) {
					$def = $1;
          # no attribution
          $def =~ s/::::.*?$//si;
          $old =~ s/::::.*?$//si;
    			&getData( $thing, "$old or $def" );
				}
				else {
					if ( $isPrivate ) {
						$output = "but $thing is $old";
					}
				}
			}
			else {
    		&getData( $thing, $def );
				if ( $isPrivate ) {
					$output = "ok, $nick";
				}
			}
		}
  }
  elsif ( $message =~ /^(.*?\S+)[\?\!]\s*$/si ) {
    my $thing = $1;
		$thing =~ s/^\s+//sgi;
		$thing =~ s/\s+$//sgi;
		$thing = lc $thing;
    my $old = &getData( $thing );
    $output = &saying( $thing, $old, $nick, $channel, $isPrivate );
	}
	elsif ( $message =~ /^\s*forget\s+(.*?)\s*$/si ) {
    my $thing = $1;
		$thing =~ s/^\s+//sgi;
		$thing =~ s/\s+$//sgi;
		$thing = lc $thing;
    my $old = &getData( $thing );
    &getData( $thing, $SILENCE );
    if ( defined $old and $old ne $SILENCE ) {
			$output = "i forgot $thing";
		}
		else {
			$output = "i didn't have anything for $thing";
		}
	}
  else {
		my @words = split /\s+/, $message;
		if ( scalar @words <= 3 ) {
      my $thing = $message;
		  $thing =~ s/^\s+//sgi;
		  $thing =~ s/\s+$//sgi;
		  $thing = lc $thing;
      my $old = &getData( $thing );
      $output = &saying( $thing, $old, $nick, $channel, $isPrivate );
    }
  }

  $irc->privmsg( $sendTo, $output ) if defined $output;

  return $output;
}

sub saying {
  my $thing = shift;
  my $old = shift;
  my $nick = shift;
  my $channel = shift;
  my $isPrivate = shift;

  return undef if ! defined $old;
  return undef if $old eq $SILENCE;

  my $output;

  my $attribute;
  my $oldchan;
  if ( $old =~ /^(.*)::::(.*?)(?:,(.*?))?$/si ) {
    $old = $1;
    $attribute = $2;
    $oldchan = $3;
  }

  $oldchan = undef if $oldchan =~ /^\s*$/si;
  if ( defined $oldchan and defined $channel and
      (lc $oldchan) ne (lc $channel) ) {
    # this is on a different channel than it was orignally
    # posted on, don't say anything
    return undef;
  }

  my $randnumber = 6;
  if ( defined $attribute ) {
    $randnumber = 9;
  }

	my $rand = int (rand $randnumber);
	$output = "i heard $thing is $old" if $rand == 0;
	$output = "like, $thing is, like, so $old" if $rand == 1;
	$output = "$thing is $old" if $rand == 2;
	$output = "someone said $thing is $old" if $rand == 3;
	$output = "$thing is $old, $nick" if $rand == 4;
  # a chance to do nothing publically
	$output = "$thing is $old, $nick" if $rand == 5 and $isPrivate;
	$output = "\"$thing is $old\" --$attribute" if $rand == 6;
	$output = "$attribute said $thing is $old" if $rand == 7;
	$output = "i heard $thing is $old from $attribute" if $rand == 8;

  if ( $old =~ /^\s*exact\s+(\S+.*)$/si ) {
    $output = $1;
  }

  return $output;
}

sub getData {
  my $name = shift;
  my $add = shift;

	my $data = OmniHash::getHash( $table );
  $$data{ $name } = $add if defined $add;
	delete $$data{ $name } if defined $add and $add eq "";
  my $retval = $$data{ $name };
	OmniHash::closeAllHashes();

  return $retval;
}

1;
