# $Id: ModQuote.pm,v 1.52 2011/11/02 20:27:57 bobzilla Exp $
# Copyright (c) 2002-2004 Phil Tom <phil@asdf.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModQuote;

@ModQuote::ISA = qw(OmniMod);

use strict;

use URI::Escape;
use HTTP::Cookies;

my $on = 1;
my $ua = new LWP::UserAgent;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(5); # 5s timeout

my $cookie_jar = HTTP::Cookies->new();
$ua->cookie_jar($cookie_jar);

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  my $output = &get_stock_quote( $message );
  $irc->privmsg( $channel, $output );
	&ModChatLog::saveMessage('omnicron', $output, $channel )
    if defined $output and $output !~ /^\s*$/si;
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  $irc->notice( $nick, &get_stock_quote( $message ) );
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^\s*quote\s+/ ) {
    return 1;
  }
  return 0;
}

sub get_stock_quote {
  my $message = shift;
  my $arg;
  my $retval = '';
  my $delta = '';
  my $percent = '';
  my $pos = 0;
  my $gt = 0;
  my $lt = 0;
  my $curquote;
  my $sign;
  my $sym;
  if ( $message =~ /^\s*quote\s*(\"?[\^\&\.\:\-\w\s]+\"?)\s*$/si ) {
    $sym = $1;
  }
  else {
    return "cannot parse '$message'";
  }
  
  # backwards compatible
  $sym = '.inx' if $sym =~ /^s&p$/i;
  $sym = '.inx' if $sym =~ /^\^?spx$/i;
  $sym = '.dji' if $sym =~ /^\^?dji$/i;

  my $qsym = uri_escape($sym);
  my $url = "http://finance.google.com/finance?q=$qsym";
  my $cboe = 0;
  my $fame = 0;

  if ( $qsym =~ /^\s*(vix|vxn|put|bxm)\s*$/si ) {
    $url = "http://www.cboe.com/DelayedQuote/SimpleQuote.aspx?TICKER=$qsym";
    $cboe = 1;

  # Try and guess if it's a person's name without running over CME symbols
  # currency pairs are shouted, and 3 long each
  # don't eat "lon:bp", mutal funds ("vfinx")
  } 
  elsif ( $sym =~ /\".*\"/s || 
      length($sym) > 5 && $sym !~ /(\d+|[A-Z]{6}|:)/s ) {
    $sym =~ s/\s+/\+/g;
    $sym =~ s/\"//g;
    $url = "http://www.quotesdaddy.com/feed/author/$sym";
    $fame = 1;
  }
  
  # make request
  my $html = &getUrl( $url );

  return &doCBOE( \$html, $sym ) if $cboe;
  return &doFamous( \$html, $sym ) if $fame;

  my $name = $sym;
  if ( $html =~ /<h3.*?>(.*?)<\/h3>/ ) {
    $name = $1;
    $name =~ s/\&amp;/\&/sgi;
    $name =~ s/\&nbsp;/ /sgi;
    $name =~ s/\&nbsp/ /sgi;
  }

  # get current price
  $html =~ s/\&nbsp;//sgi;

  if ($html =~ /<span class="?pr"?.*?>\s*(?:<span id="[^>]*>)?\s*(.+?)\s*<\/span>/si ) {
    $curquote = $1;
    $curquote =~ s/<span.*?>//si;
  }
  else {
    # try cme
    return &doCME( $sym );
  }

  # get delta
  if ($html =~ /<span class\s*=\s*"ch[rg](?: bld)?"[^>]*?>\s*(.+?)\s*<\/span>\s*<\/span>\s*<span class="ch[rg]"[^>]*?>\((.*?)\)/si ) {
    $delta = $1;
    $percent = $2;
  }
  elsif ($html =~ /<span class\s*=\s*"[\w\s]*ch[rg][\w\s]*"[^>]*?>\s*(.+?)\s*\((.*?)\)\s*<\/span>/si ) {
    $delta = $1;
    $percent = $2;
    $delta =~ s/<.*?>//sgi;
    $percent =~ s/<.*?>//sgi;
  }

  my $prepost;
  if ($html =~ />(Pre-Market:|After Hours:)\s*<span.*?>(.*?)<\/span>\s*<span.*?>(.*?)<\/span>.*?(\(.*?\))<\/span>/si ) {
    $prepost = "$1 $2 $3 $4";
  }

  my %keys = ();
  my @pairs = split /class="key"\s*data-snapfield=.*?>/si, $html;
  foreach my $pair ( @pairs ) {
    if ( $pair =~ /^\s*(.*?)\s*<\/td>\s*(?:<td.*?>)?\s*(.*?)\s*(?:<\/td>)/si ) {
      my $key = $1;
      my $val = $2;
      $key =~ s/:$//;
      # print STDERR "key '$key' val '$val'\n";
      $keys{$key} = $val;
    }
  }

  my $range = '';
  my $vol = '';
  my $week52 = '';
  my $div = '';

  $range = $keys{'Range'} if defined $keys{'Range'};
  $vol = $keys{'Vol / Avg.'} if defined $keys{'Vol / Avg.'};
  $week52 = $keys{'52 week'} if defined $keys{'52 week'};
  $div = $keys{'Div/yield'} if defined $keys{'Div/yield'};

  $retval = $name . " | $curquote $delta ($percent)";
  if ( $range ne '' and $range ne '-' ) {
    $retval .= " | $range";
  }
  if ( $vol ne '' ) {
    $retval .= " | $vol ";
  }
  if ( $week52 ne '' ) {
    $retval .= " | 52wk: $week52";
  }
  if ( $div ne '' and $div ne '-' ) {
    $retval .= " | Div/yield: $div";
  }
  if ( defined $prepost ) {
    $retval .= " | $prepost";
  }
  $retval =~ s/\s+/ /sgi;

  return $retval;
}

sub name() {
  my $this = shift;
  return "quote";
}

sub help {
    if ( defined wantarray ) { # do we have help?
        return 1;
    }

    my ($thisi,$irc,$nick,$message) = @_;

    $irc->notice( $nick, "quote <symbol> -> report stock info for symbol. You can prepend <symbol> with '*' to force a \"market\" rather than \"stock\" lookup." );
    $irc->notice( $nick, "quote <author> -> get a quote from author" );
}

sub doFamous {
	my $htmlptr = shift;
	my $sym = shift;

	my $q; my $a;

	my $startRE = ".*item>.*description>";
	my $endRE = "<\/description>.*";

	if ( $$htmlptr !~ /item/si ) {
		return "$sym ?";
	} elsif ( ($q, $a) = ( $$htmlptr 
	    =~ /^$startRE(\&quot\;.*\&quot\;)\s+\-\s+(.*)$endRE$/sio) ) {
	    	
		$q = uri_unescape($q);
		$q =~ s/\&quot\;/\"/g;

		if ( $sym =~ /confucius/ ) {
			return "$a say $q";
		} else {
			return "$q --$a";
		}

	} else {
		return "$sym?";
	}
}

sub doCBOE {
  my $htmlptr = shift;
  my $sym = shift;
  
  if ( $$htmlptr =~ /Quote Data Start -->(.*?)<\!-- Price Data Table/si ) {
    my $price = $1;
    $price =~ s/<.*?>//sg;
    $price =~ s/[\n\r]//sg;
    $price =~ s/\&nbsp;/ /sg;
    $price =~ s/\s+/ /sg;
    $price =~ s/^\s+//sg;
    $price =~ s/\s+$//sg;
    $price =~ s/(\s+\S+\s+\S+)$/ |$1/sg;

    my $prevclose = '';
    if ( $$htmlptr =~ /Previous Close\s*<.*?>\s*<.*?>\s*(.*?)\s*</si ) {
      $prevclose = $1;

      if ( $price =~ /\s+(\S+)$/si ) {
        my $change = $1;
        my $percent = 100 * $change / $prevclose;
        $percent =~ s/(\.\d\d).*$/$1/si;
        $price .= " ($percent\%)";
      }
    }

    my $high = '';
    my $low = '';
    if ( $$htmlptr =~ /High\s*<.*?>\s*<.*?>\s*(.*?)\s*</si ) {
      $high = $1;
    }
    if ( $$htmlptr =~ /Low\s*<.*?>\s*<.*?>\s*(.*?)\s*</si ) {
      $low = $1;
    }

    my $high52 = '';
    my $low52 = '';
    if ( $$htmlptr =~ /52 Week High\s*<.*?>\s*<.*?>\s*(.*?)\s*</si ) {
      $high52 = $1;
    }
    if ( $$htmlptr =~ /52 Week Low\s*<.*?>\s*<.*?>\s*(.*?)\s*</si ) {
      $low52 = $1;
    }
    return "$price | $high $low | 52wk: $high52 $low52";
  }
  else {
    return "$sym?";
  }
}

sub getUrl {
  my $url = shift;
  my $request = HTTP::Request->new(GET => $url);
  my $response = $ua->request($request);
  my $html = $response->as_string();
  return $html;
}

sub doCME {
  my $sym = shift;
  
  # start by supporting simple base case
  if ( $sym =~ /^(\w{2,3})\w\d$/ ) {
    my $product = uc $1;
    my $currenttime = time * 1000;
    my $reqsym = uri_escape( uc $sym );
    my $url = "http://cmegroup.com/CmeWS/md/MDServer/V1/Venue/G/Exchange/XCME/FOI/FUT/Product/$product?currentTime=$currenttime&contractCDs=$reqsym";
    # make request
    my $html = &getUrl( $url );
    # print STDERR "html: $html\n";
    if ( $html =~ /"tradePrice":"(.*?)"/si ) {
      my $lasttrade = $1;
      my $retval = $reqsym . " | " . $lasttrade;

      if ( $html =~ /"netChange":"(.*?)"/si ) {
        my $netChange = $1;
        $retval .= " ";
        $retval .= "+" if ( $netChange ne '-' and $netChange > 0 );
        $retval .= "$netChange";
      }
      $retval .= " |";

      my $highPrice;
      my $lowPrice;
      if ( $html =~ /"highPrice":"(.*?)"/si ) {
        $highPrice = $1;
      }
      if ( $html =~ /"lowPrice":"(.*?)"/si ) {
        $lowPrice = $1;
      }
      if ( defined $highPrice and defined $lowPrice ) {
        $retval .= " $lowPrice - $highPrice |";
      }

      if ( $html =~ /"cumulativeVolume":"(.*?)"/si ) {
        my $volume = $1;
        $retval .= " $volume |";
      }

      if ( $html =~ /"openInterest":"(.*?)"/si ) {
        my $openInterest = $1;
        $retval .= " openInterest: $openInterest |";
      }

      return $retval;
    }
  }
  
  return "$sym?";
}

1;
