# $Id: ModWord.pm,v 1.4 2010/08/25 17:26:57 uqbar Exp $
# Copyright (c) 2002-2004 Phil Tom <phil@asdf.com>
#
# ModWord.pm.pm - Dictionary lookup
# Author: uqbar
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModWord;

@ModWord::ISA = qw(OmniMod);

use strict;

use URI::Escape;
use HTTP::Cookies;

my $on = 1;
my $ua = new LWP::UserAgent;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(5); # 5s timeout

my $cookie_jar = HTTP::Cookies->new();
$ua->cookie_jar($cookie_jar);

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  my $output = &get_meaning( $message );
  $irc->privmsg( $channel, $output );
	&ModChatLog::saveMessage('omnilearn', $output, $channel )
    if defined $output and $output !~ /^\s*$/si;
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  $irc->notice( $nick, &get_meaning( $message ) );
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^\s*word\s+/ ) {
    return 1;
  }
  return 0;
}

sub get_meaning {
	my $message = shift;
  	my $arg;
  	my $pos = 0;
  	my $word;
  	if ( $message =~ /^\s*word\s*(.*?)\s*$/si ) {
	  $word = $1;
	  $word =~ s/\s/\+/g;

          my $url = "http://services.aonaware.com/DictService/DictService.asmx/DefineInDict?dictId=wn&word=$word";
          #print "Using URL $url\n";
          # make request
          my $html = &getUrl( $url );
          $html =~ tr/\r\n/ /d;

          if( $html !~ m/<Definition>/ ) {
            return "Not sure what '$word' means";
          } elsif( $html =~ m/<WordDefinition>(.*?)<\/WordDefinition>/i ) {
            my $temp=$1;
            $temp =~ s/\s{9}//g;
            return "--> $temp\n";
          } else {
            return "Not sure what '$word' means";
          }
       }
}

sub name() {
  my $this = shift;
  return "word";
}

sub help {
    if ( defined wantarray ) { # do we have help?
        return 1;
    }

    my ($thisi,$irc,$nick,$message) = @_;

    $irc->notice( $nick, "word <word to lookup> -> gets the first dictionary meaning of the word." );
}

sub getUrl {
  my $url = shift;
  my $request = HTTP::Request->new(GET => $url);
  my $response = $ua->request($request);
  my $html = $response->as_string();
  return $html;
}


1;
